<?php
		
/* Class: fakeRs (it means fakeRecordset)
 * Description:
 * 		This class keep inside all the information required by the shopping kart. (fake Recordset)
 * 		It simulate a ADOdb Recordset to an imaginary db, but all the informations are not saved into any db.
 * 		Also you have the posibility to add, remove, see the items in this fake db
 * Extends: Nothing	
 */
	
class krt_fakeRS {
		// In the fields are all the columns and the rows for this fake recordset.
	var $fields=array();
		// The current row
	var $currRow = 0;
		// The total number of rows
	var $nRows = 0;
		// END OF FILE
	var $EOF = true;
		// BEGIN OF FILE
	// This is never used ...!?
		var $BOF = true;
		//Page Size for Recordset Navigation
		var $pageSize;
		var $pageStart;
	
	/* Function: 
	 * updateOne
	 * Description:
	 * 	  Update the value which is kept at the $n-th row, in the column $field.
	 * Parameters:
	 *    $n - The row where the new value will be placed
	 *    $field - The field from the row where the value should arrive.
	 *    $value - The new value of the matrix
	 * Returns:
	 * 		nothing
	 */
	function updateOne($n, $field, $value){
	  if ( $n >= $this->nRows || $n < 0) {
	    die("Index out of range");
	  }
	  if( $value == "NULL"){
	    $value="";
	  }
	  if (isset($this->fields[$n][$field])){
					$this->fields[$n][$field] = $value;
			}else{
					die("The field $field is not in the list of fields!");	
			}
	}
	/* Function: updateFields
	 * Description:
	 * 	 Update one or multiple values from the same row. 
	 * 	 If only one row it will be updated the update1 function is used. 
	 * Parameters:
	 *    $n - The row which will be updated
	 *    $fields - Associative array -  The field(s) that should be updated
	 * Returns:
	 * 		nothing
	 */
		
	function updateFields($n, $fields){
		foreach ($fields as $key=>$value){
			$this->updateOne($n, $key, $value);							
		}
	}
	/* Function: insertFields
	 * Description:
	 * 		Insert a new row at the end of the rows. If the EOF was true it became false. 
	 * 		The total number of rows is increased by 1.
	 * Parameters:
	 *    $fields - The fields for this rows (This are not restricted, it doesn't metter if they exist)
	 * Returns:
	 * 		nothing
	 */
	function insertFields($fields){
	  $this->fields[$this->nRows] = array();
	  $cr = $this->nRows++;
	
	  foreach ($fields as $key=>$value) {
	    $this->fields[$cr][$key]=$value;
	  }
	  $this->EOF = false;
	}
	/* Function: deleteFields
	 * Description:
	 * 	  Delete the $n-th row.
	 * Parameters:
	 *    $n - The row number to be deleted.
	 * Returns:
	 * 		nothing
	 * Bug: If the current row is in the $n, $nRows-1 it will be automatically point to the currRows++ 
	 * 			(But the case in which currRows++ became the last is treat for the EOF)
	 */
	function deleteFields($n) {
	  if ( $n >= $this->nRows || $n < 0 ) {
				return;
	  }
	  for ( $i = $n+1; $i < $this->nRows; $i++) {
	    $this->fields[$i-1] = $this->fields[$i];
	  }
	  array_pop($this->fields);
	  $this->nRows--;
	  if ($this->currRow == $this->nRows) {
	    $this->currRow--;
	  }
			if ($this->currRow == $this->nRows-1) {
	     $this->EOF = true;
	  }
	}
	/* Function: MoveNext
	 * Description: 
	 * 		Move to the next row in the list of rows if it can.
	 * 		The BOF should be set on false if not the first row.
	 * Parameters:
	 * 		None
	 * Returns:
	 * 		true - if the move action succeded
	 * 		false - if no more rows in the Recordset
	 * Bug: 
	 * 		The EOF flag is not set true if the currRow arrive at the end.
	 */
	
	function MoveNext(){
		//if the recordset is paged
		if (isset($this->pageSize) && $this->pageSize) {
			if (($this->pageStart + $this->pageSize - $this->currRow - 1 > 0)
					&& ($this->nRows - $this->currRow - 1 > 0)) {
				$this->currRow++;
				return true;
			}
			$this->EOF = true;
			return false;
		}
		//if not
		if ($this->nRows - $this->currRow - 1 > 0) {
			$this->currRow++;
			return true;
		}
		$this->EOF = true;
		return false;
	}
	/* Function: MoveFirst
	 * Description:
	 * 		Move the current row to the first row in the recordset.
	 * 		The BOF should be set on true
	 * Parameters:
	 * 		none
	 * Returns:
	 * 		nothing
	 */
	function MoveFirst(){
	  $this->currRow = 0;
	  if ($this->nRows > 0) {
	    $this->EOF = false;
	  } else {
	    $this->EOF = true;
	  }
	}
		
	/* Function: Fields
	 * Description:
	 * 		Retrive the value stored in the $field on the current row.
	 * Parameters:
	 *    $field - The field for which the value will be get it.
	 * Returns:
	 * 		The value from the $field in the current row.
	 * BUG 
	 * 		If the field don't exist an exception should be throw.
	 */
	
	function Fields($field){
	  return @$this->fields[$this->currRow][$field];
	}
		/* Function: close
		 * Description:
		 * 		Close the recordset unsetting all the values contained here
		 * Parameters:
		 *    none	
		 * Returns:
		 * 		nothing
		 */
	
	function Close(){
	}
	
		/**
			get the number of rows in the recordset
			@params
				none
			@return
				none		
		*/
		function RecordCount() {
			return $this->nRows;
		}
		
		/**
			move the recordset index to specified position
			affect BOF and EOF
			@param
				$idx - desired index
			@return
				none
		*/
		function Move($idx) {
			//if we move on the last position or beyond
			if ($idx >= ($this->nRows)) {
	    $this->currRow = $this->nRows - 1;
				$this->EOF = true;
				return;
			}
			//if we move on the first position or before
			if ($idx <= 0) {
				$this->MoveFirst();
				return;
			}
			$this->currRow = $idx;
		}
		
		/**
			sets the pageSize of the recordset
			@params
				$pageStart - index where Page Starts
				$pageSize	 - page size
			@return
				none
		*/
		function setPage($pageStart,$pageSize) {
			$this->pageStart = $pageStart;
			$this->pageSize = $pageSize;
			$this->Move($this->pageStart);
		}

					
} // End class fakeRs
?>
