<?php

	class krt_mxKart extends krt_fakeRS {
		
		var $fieldsName = array();
		var $priceCol;
		var $origPriceCol;
		var $origPricePropsCol;
		var $quantityCol;
		var $fp;
		var $unique = array();
		var $specialProps = array();
		var $orderID;
		
		/**
			Constructor
			Copy from kart Properties the Kart Fields Names
		*/
		function krt_mxKart() {
			$tmp = uniqid("", true);
			$this->orderID = preg_replace("/([^\d])/e", "(ord('$1')-ord('.'))", $tmp);
			$_SESSION['KT_kartOrderId'] = $this->orderID;
		}
		
		/**
			include special discount classes (if exists)
			@params
				$discountList - list woth the discounts
			
			@return
				none
		*/
		function requireDiscounts($discountList) {
			foreach($discountList as $key=>$value) {
				@include_once(MXK_DIR.'/../../includes/krt/MXKD' . ucfirst($value) . '.class.php');
			}
		}

		function requireFiles($prefix, $fileList, $eee=false) {
			ob_start();
			foreach($fileList as $key=>$value) {
				include_once(MXK_DIR . DIRECTORY_SEPARATOR . $prefix . ucfirst($value) . '.class.php');
			}
			ob_end_clean();
		}

		function loadProps() {
			$kartProps = $GLOBALS['kartProps'];
			
			$this->fieldsName = $kartProps['fieldProps'];
			$this->priceCol = $kartProps['price'];
			$this->origPriceCol = 'origPrice';
			$this->origPricePropsCol= 'origPriceProps';
			$this->quantityCol = $kartProps['quantity'];
			//$this->ft = $kartProps['ft'];
			$this->unique = $kartProps['id'];
			$this->specialProps = $kartProps['specialProps'];
			$this->additiveFields = $kartProps['additiveFields'];
			$this->Korders = $kartProps['Korders'];
			if ($kartProps['shipping']) {
				$this->shipping = $kartProps['shipping'];
				ob_start();
				require_once(MXK_DIR.'/../../' . $this->shipping['file']);
				ob_end_clean();
				$this->requireFiles('MXKR', $this->shipping['rates']);
			}
			if ($kartProps['taxes']) {
				$this->taxes = $kartProps['taxes'];
				ob_start();
				require_once(MXK_DIR.'/../../' . $this->taxes['file']);
				ob_end_clean();
				$this->requireFiles('MXKT', $this->taxes['rates']);
			}
			if ($kartProps['discount']) {
				$this->discount = $kartProps['discount'];
				ob_start();
				require_once(MXK_DIR.'/../../' . $this->discount['file']);
				ob_end_clean();
				$this->requireFiles('MXKD', $this->discount['rates'], 1);
				$this->fp = $this->discount['rates'];
			}
		}
		
		
		function getOrderID() {
			return $this->orderID;
		}
		
		/**
			adds a new kart element
			@params
				$fields - Associative Array - fields to be inserted
			@return
				none
		*/
		function addToKart($fields) {
			$fields[$this->quantityCol] = abs((int)($fields[$this->quantityCol]));
			//if the new field exists in the kart recordset then add the new quantity to old quantity
			$myUniqueId = $this->getUniqueId($fields);
			for ($rowIdx = 0;$rowIdx < $this->nRows;$rowIdx++) {
				if ($this->getUniqueId($rowIdx) == $myUniqueId) {
					$this->updateOne($rowIdx,$this->quantityCol,$this->fields[$rowIdx][$this->quantityCol] + $fields[$this->quantityCol]);
					//$this->updateOne($rowIdx,$this->priceCol,$fields[$this->priceCol]);
					$this->UpdateKart();
					return;
				}
			}
			//if not insert the new field
			$this->insertFields($fields);
			$this->UpdateKart();
		}
		
		/**
			return the Price of all buyed items
			@params
				none
			@return
				total price
		*/
		function getTotalPrice($discounted) {
			$totalPrice = 0;
			for ($idx = 0;$idx < $this->nRows;$idx++) {
				$totalPrice += $this->getSubTotal($idx, $discounted);
			}
			return $totalPrice;
		}
		
		/**
			apply the discounts per each product(row in the recordset) and return the price
			@params
				$idx - row for which we compute the dicounted price
				$initialPrice - initial price (departing point)
			@return 
				new price
		*/
		function getFPPrice($initialPrice,$uniqueKeyVal) {
			$ret = $initialPrice;
			if ($uniqueKeyVal=='') {
				return $ret;
			}
			$lastFunctions = array();
			
			for ($idxFP = 0;$idxFP < count($this->discount['rates']);$idxFP++) {
				$funcName = 'MXKDiscountFp_' . $this->fp[$idxFP];
				switch(strtolower($this->fp[$idxFP])) {
					case 'volumediscount':
						// do not apply the volume discount yet
						$lastFunctions[] = $this->fp[$idxFP];
						break;
					default:
						// apply non-total discounts
						
						if (function_exists($funcName)) {
							
							if ($this->discount['applyMin'] == '1') {
								$tmP = $funcName($initialPrice,$uniqueKeyVal);
								if ($tmP < $ret) {
									$ret = $tmP;
								}
							} else {
								$ret = $funcName($ret,$uniqueKeyVal);
							}
						}
				}
			}
			// cache price without volume discount for the current product
			for ($i = 0;$i < count($this->fields);$i++) {
				if ($this->fields[$i][$this->unique] == $uniqueKeyVal) {
					$this->fields[$i]['priceColWoVol'] = $ret;
				} else {
					if (!isset($this->fields[$i]['priceColWoVol'])) {
						$this->fields[$i]['priceColWoVol'] = $this->fields[$i][$this->priceCol];
					}
				}
			}
			// finally, apply the volume discount
			for ($idxFP = 0;$idxFP < count($lastFunctions);$idxFP++) {
				$funcName = 'MXKDiscountFp_' . $lastFunctions[$idxFP];
				if (function_exists($funcName)) {
					if ($this->discount['applyMin'] == '1') {
						$tmP = $funcName($initialPrice,$uniqueKeyVal);
						if ($tmP < $ret) {
							$ret = $tmP;
						}
					} else {
						$ret = $funcName($ret,$uniqueKeyVal);
					}
				}
			}
			
			return $ret;
		}
		
		/**
			apply the final discounts over the total price
			@params
				$initialPrice - initial price (departing point)
			@return
				new price
		*/
		function getFTPrice($initialPrice = 0) {
			$ret = $initialPrice;
			
			for ($idxFT = 0;$idxFT < count($this->ft);$idxFT++) {
				$funcName = 'MXKDiscountFt_' . $this->ft[$idxFT];
				if (function_exists($funcName)) {
					$ret = $funcName($ret,$this);
				}
			}
			return $ret;
		}

		/**
			get the total cost of shipping
			@params
				none
			@return
				shipping costs
		*/
		function getShippingCost() {
			$shipCost = (float)0;
			for ($idxRate = 0;$idxRate < count($this->shipping['rates']);$idxRate++) {
				$funcName = 'MXKRate_' . $this->shipping['rates'][$idxRate];
				if (function_exists($funcName)) {
					$shipCost = $funcName($shipCost);
				} 
			}
			return $shipCost;
		}
		
		/**
			get the taxes amount
			@param
			@return
				amount of taxes
		*/
		function getTaxes($rowIdx) {
			$tax = (float)0;
			for ($idxTax = 0;$idxTax < count($this->taxes['rates']);$idxTax++) {
				$funcName = 'MXKTax_' . $this->taxes['rates'][$idxTax];
				if (function_exists($funcName)) {
					$tax = $funcName($tax,$rowIdx);
				}
			}
			return $tax;
		}
		/**
			get the sum of all taxes
		*/
		function getTotalTaxes() {
			$tax = (float)0;
			for ($rowIdx = 0;$rowIdx < $this->nRows;$rowIdx++) {
				$tax += $this->getTaxes($rowIdx);
			}
			return $tax;
		}

		
		/**
			get an unique identifier for a row
			@params
				$idRow - either Row number (integer), either an associtive array with row fields
				
			@return
				string - unique identifier for a row
		*/
		function getUniqueId($idRow) {
			if (is_array($idRow)) {
				$fields = $idRow;
				$props = $fields['properties'];
			} else {
				$fields = $this->fields[$idRow];
				$props = $fields['properties'];
			}
			$uid = '';
			$fieldIdx = 0;
			$uid = $fields[$this->unique] . '_' . md5($props);
			return $uid;
		}
		
		/**
			get the row index that match a specific uid
			@params
				$uid - string - Unique Identifier for a row
			@return
				- integer - row index or -1 if not found
		*/
		function getIndexForUID($uid) {
			$fieldIdx = 0;
			for ($fieldIdx = 0;$fieldIdx < $this->nRows;$fieldIdx++) {
				if ($uid == $this->getUniqueId($fieldIdx)) {
					return $fieldIdx;
				}
			}
			return -1;
		}
		
		
		/**
			get the total for Optional Additive Properties
			@params
				$addProp - Assoc Array - Optional Additive Property object	
							- fieldName
							- columnName
			@return
				float - total value
		*/
		function getAdditiveTotal($addProp) {
			$total = 0;
			for ($idx = 0;$idx < $this->nRows;$idx++) {
				$total += (float)@$this->fields[$idx][$addProp['fieldName']] * (float)@$this->fields[$idx][$this->quantityCol];
			}
			return $total;
			
		}
		/**
			get the name for the Total Field of an Optional Additive Property
			@params
				$fieldName - the name of the additive field
			@return
				total field name
		*/
		function totalAddName($fieldName) {
			return 'MXK_Total_' . $fieldName;
		}
		
		/* Function: Fields
		 * Description:
		 * 		Retrive the value stored in the $field on the current row or call the special function for special fieds
		 * Parameters:
		 *    $field - The field for which the value will be get it.
		 * Returns:
		 * 		The value from the $field in the current row.
		 * BUG 
		 * 		If the field don't exist an exception should be throw.
		 */
		function Fields($field) {
			for ($propIdx = 0;$propIdx < count($this->additiveFields);$propIdx++) {
				if ($this->totalAddName($this->additiveFields[$propIdx]['fieldName']) == $field) {
					return $this->getAdditiveTotal($this->additiveFields[$propIdx]);
				}
			}
			switch ($field) {
				case 'MXK_TotalPrice':
					return krt_showPrice($this->getTotalPrice(true));
				break;
				case 'MXK_TotalUndisc':
					return krt_showPrice($this->getTotalPrice(false));
				break;
				case 'MXK_UID':
					return $this->getUniqueId($this->currRow);
				break;
				case 'MXK_SUBTOTAL':
					return krt_showPrice($this->getSubTotal($this->currRow));
				break;
				case 'MXK_Shipping':
					return krt_showPrice($this->getShippingCost());
				break;
				case 'MXK_Taxes':
					return krt_showPrice($this->getTaxes($this->currRow));
				break;
				case 'MXK_TaxesTotal':
					return krt_showPrice($this->getTotalTaxes());
				break;
				case 'MXK_TotalPayable':
					return krt_showPrice($this->getTotalPayable());
				break;
				case 'price':
					return krt_showPrice(parent::Fields($field));
				break;
				default:
					return parent::Fields($field);
			}
		}
		
		function GetRowAssoc() {
			$tm = array();
			foreach ($this->fieldsName as $key=>$value) {
				$tm[$value] = 1;
			}
			foreach ($this->specialProps as $key=>$value) {
				$tm[$value] = 1;
			}
			foreach ($this->additiveFields as $key=>$value) {
				$tm[$this->totalAddName($value['fieldName'])] = 1;
			}
			return $tm;
		}
		/**
			get the total Payable Amount
		*/
		function getTotalPayable() {
			return $this->getTotalPrice(true) + $this->getTotalTaxes() + $this->getShippingCost();
		}
		
		/**
			get the subtotal for a row
			@params
				$idx - row index
				$discounted - boolean - get discounted price
			@return 
				the subtotal (quantity * price)
		*/
		function getSubTotal($idx, $discounted=true) {
			if($discounted) {
				$uPrice = $this->fields[$idx][$this->priceCol];
			} else {
				$uPrice = $this->fields[$idx][$this->origPriceCol];
				if(isset($this->fields[$idx][$this->origPricePropsCol])) {
					$uPrice += $this->fields[$idx][$this->origPricePropsCol];
				}
			}
			$quantity = $this->fields[$idx][$this->quantityCol];
			return $uPrice*$quantity;
		}

		/**
			Updates the prices for each product in the kart. This function should be called when users loggs in / out
			@params
				none
			@return
				nothing
		*/
		function UpdateKart() {
			$i = 0;
			while (@$this->fields[$i]) {
				$this->fields[$i][$this->priceCol] = $this->getFPPrice($this->fields[$i][$this->origPriceCol],$this->fields[$i][$this->unique]) + @$this->fields[$i][$this->origPricePropsCol];
				$i++;
			}
		}

		function prefillOrderDetails($userId, &$tNG) {
			$kartProps = $GLOBALS['kartProps'];
			if (isset($userId)) {
				$sql = "SELECT ".$kartProps['Korders']['orders']['firstname']." AS col1 FROM ".$kartProps['Korders']['orders']['tableName']." WHERE ".$kartProps['Korders']['orders']['pk']."='".$this->getOrderID()."'";
				$rs = $tNG->connection->Execute($sql) or die($tNG->connection->ErrorMsg());
				
				if ($rs->Fields('col1') != '') {
					return;
				}
				
				$sql = "SELECT ".$kartProps['Korders']['orders']['firstname'].", ".$kartProps['Korders']['orders']['lastname'].", ".$kartProps['Korders']['orders']['email'].", ".$kartProps['Korders']['orders']['phone'].", ".$kartProps['Korders']['orders']['streetaddress'].", ".$kartProps['Korders']['orders']['city'].", ".$kartProps['Korders']['orders']['state'].", ".$kartProps['Korders']['orders']['country'].", ".$kartProps['Korders']['orders']['zip'].", shipmethod_ord, shipname_ord,  shipstreetaddress_ord, shipcity_ord, shipstate_ord, shipcountry_ord, shipzip_ord FROM ".$kartProps['Korders']['orders']['tableName']." WHERE ".$kartProps['Korders']['orders']['pk']."!='".$this->getOrderID()."' AND ".$kartProps['Korders']['orders']['userid']."=$userId ORDER BY ".$kartProps['Korders']['orders']['date']." DESC LIMIT 1";
				$rs = $tNG->connection->Execute($sql) or die($tNG->connection->ErrorMsg());
				$sql = "";
				if ($rs && $rs->RecordCount() > 0){
					foreach($rs->fields as $key=>$value){
						if (!preg_match("/^\d*$/", $key)) {
							if ($sql != '') {
								$sql .= ', ';
							}
							$sql .= $key . "='" . addslashes($value) . "'";
						}
					}
				}
				if ($sql!='') {
					$sql = 'UPDATE '.$kartProps['Korders']['orders']['tableName'].' SET ' . $sql . ' WHERE '.$kartProps['Korders']['orders']['pk'].' =\'' . $this->getOrderID() . '\'';
					$tNG->connection->Execute($sql) or die($tNG->connection->ErrorMsg());
				}
			} else {
				//return new tNG_error('');
			}
			return '';
		}
	}

?>